@@ -26,6 +26,7 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+import pycopy
 from pycopy import const
 import sys
 import ffilib
@@ -73,7 +74,11 @@ _UNESCAPE_DICT = const({
 
 
 class error(Exception):
-    pass
+
+    def __init__(self, msg, pattern=None, pos=None):
+        self.msg = msg
+        self.pattern = pattern
+        self.pos = pos
 
 
 class PCREMatch:
@@ -205,7 +210,8 @@ class PCREPattern:
         res = b""
         pos = 0
         cnt_rpl = 0
-        while True:
+        finish = len(s)
+        while pos <= finish:
             m = self.search(s, pos)
             if not m:
                 res += s[pos:]
@@ -266,9 +272,10 @@ class PCREPattern:
                     break
 
         if is_str:
-            for x in res:
+            for i in range(len(res)):
+                x = res[i]
                 if x is not None:
-                    x.__class__ = str
+                    res[i] = pycopy.icast(x, str)
 
         return res
 
@@ -301,12 +308,12 @@ class PCREPattern:
                 pos += 1
 
         if is_str:
-            for x in res:
+            for i in range(len(res)):
+                x = res[i]
                 if isinstance(x, tuple):
-                    for x1 in x:
-                        x1.__class__ = str
+                    res[i] = tuple([pycopy.icast(x1, str) for x1 in x])
                 else:
-                    x.__class__ = str
+                    res[i] = pycopy.icast(x, str)
 
         return res
 
@@ -397,12 +404,21 @@ def finditer(pattern, s, flags=0):
 
 
 def escape(s):
-    res = ""
-    for c in s:
-        if '0' <= c <= '9' or 'A' <= c <= 'Z' or 'a' <= c <= 'z' or c == '_':
-            res += c
-        else:
-            res += "\\" + c
+    if isinstance(s, str):
+        res = ""
+        for c in s:
+            if '0' <= c <= '9' or 'A' <= c <= 'Z' or 'a' <= c <= 'z' or c == '_':
+                res += c
+            else:
+                res += "\\" + c
+    else:
+        res = b""
+        for c in s:
+            c = bytes([c])
+            if b'0' <= c <= b'9' or b'A' <= c <= b'Z' or b'a' <= c <= b'z' or c == b'_':
+                res += c
+            else:
+                res += b"\\" + c
     return res
 
 
